#include "SDL.h"
#include <math.h>

#include "globals.h"
#include "constants.h"

void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
}

Uint32 getpixel(SDL_Surface *surface, int x, int y)
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;

    case 2:
        return *(Uint16 *)p;

    case 3:
		if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            return p[0] << 16 | p[1] << 8 | p[2];
		}
		else {
            return p[0] | p[1] << 8 | p[2] << 16;
		}

    case 4:
        return *(Uint32 *)p;

    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
}

void apply_surface(SDL_Surface* src, SDL_Surface* dst, int x, int y) {
	SDL_Rect dstRect;

	dstRect.x = x;
	dstRect.y = y;

	SDL_BlitSurface(src, NULL, dst, &dstRect);
}

void fade_screen(int ballX, int ballY, int radius) {
	SDL_LockSurface(screen);

	Uint8 r, g, b;
	Uint32 pixelClr;

	int lBorder = ballX - radius;
	int rBorder = ballX + radius;
	int tBorder = ballY - radius;
	int bBorder = ballY + radius;

	if (lBorder < 0) {lBorder = 0;}
	if (rBorder > SCR_WIDTH) {rBorder = SCR_WIDTH;}
	if (tBorder < 0) {tBorder = 0;}
	if (bBorder > SCR_HEIGHT) {bBorder = SCR_HEIGHT;}
	
	for (int x = lBorder; x < rBorder; x++) {
		for (int y = tBorder; y < bBorder; y++) {
			pixelClr = getpixel(screen, x, y);
			SDL_GetRGB(pixelClr, screen->format, &r, &g, &b);

			if (r > FADE_STEP - 1) {r -= FADE_STEP;}
			if (g > FADE_STEP - 1) {g -= FADE_STEP;}
			if (b > FADE_STEP - 1) {b -= FADE_STEP;}

			if (r < FADE_STEP) {r = 0;}
			if (g < FADE_STEP) {g = 0;}
			if (b < FADE_STEP) {b = 0;}

			pixelClr = SDL_MapRGB(screen->format, r, g, b);
			putpixel(screen, x, y, pixelClr);
		}
	}

	SDL_UnlockSurface(screen);
}

void draw_field() {
	bool check = false;
	int dashHeight = SCR_HEIGHT / DASH_AMOUNT;
	SDL_Rect tRect;

	tRect.w = DASH_WIDTH;
	tRect.h = dashHeight;
	tRect.x = (SCR_WIDTH / 2) - (DASH_WIDTH / 2);

	for (int x = 0; x <= DASH_AMOUNT / 2; x++) {
		if (check) {
			tRect.y = x * dashHeight * 2;
			SDL_FillRect(screen, &tRect, 0x00FFCC);
		} else {
			check = true;
		}
	}

	tRect.w = SCR_WIDTH;
	tRect.h = 2;
	tRect.x = 0;
	tRect.y	= 38;

	SDL_FillRect(screen, &tRect, 0x00FFCC);
	
	// Draw the AI On/Off Indicators
	tRect.x = 8;
	tRect.y = 8;
	tRect.w = 19;
	tRect.h	= 19;
	SDL_FillRect(screen, &tRect, 0x999999);
	tRect.x = SCR_WIDTH - 8 - 19;
	SDL_FillRect(screen, &tRect, 0x999999);
	
	tRect.x = 11;
	tRect.y = 11;
	tRect.w = 13;
	tRect.h = 13;
	if (p1Bat.ai_enabled()) {
		SDL_FillRect(screen, &tRect, 0x00FF00);
	} else {
		SDL_FillRect(screen, &tRect, 0xFF0000);
	}

	tRect.x = SCR_WIDTH - 11 - 13;
	if (p2Bat.ai_enabled()) {
		SDL_FillRect(screen, &tRect, 0x00FF00);
	} else {
		SDL_FillRect(screen, &tRect, 0xFF0000);
	}
}

bool rect_intersect(SDL_Rect* rect1, SDL_Rect* rect2) {
	if (rect1->x <= rect2->x + rect2->w && rect1->x + rect1->w >= rect2->x) {
		if (rect1->y <= rect2->y + rect2->h && rect1->y + rect1->h >= rect2->y) {
			return true;
		}
	}

	return false;
}

float calc_angle(SDL_Rect* rect1, SDL_Rect* rect2) {
	float midX1 = rect1->x + .5 * rect1->w;
	float midY1 = rect1->y + .5 * rect1->h;

	float midX2 = rect2->x + .5 * rect2->w;
	float midY2 = rect2->y + .5 * rect2->h;

	return atan2(midY2 - midY1, midX2 - midX1) * RAD_TO_DEG;
}