#include "SDL.h"
#include "SDL_ttf.h"
#include <string>
#include <sstream>
#include <math.h>
#include <time.h>

#include "timer.h"
#include "bat.h"
#include "ball.h"
#include "constants.h"
#include "globals.h"
#include "functions.h"
#include "score.h"

int init() {
	// Start up SDL, SDL's TTF library, load the screen, load fonts and create some colors
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) == -1) {
		return 1;
	}

	screen = SDL_SetVideoMode(SCR_WIDTH, SCR_HEIGHT, SCR_BPP, SDL_SWSURFACE);
	if (screen == NULL) {
		return 1;
	}

	if (TTF_Init() < 0) {
		return 1;
	}
	
	SDL_WM_SetCaption("Pong", NULL);

	scoreFont = TTF_OpenFont("arial.ttf", 100);
	defaultFont = TTF_OpenFont("arial.ttf", 30);

	clrWhite = SDL_MapRGB(screen->format, 255, 255, 255);
	clrBlack = SDL_MapRGB(screen->format,   0,   0,   0);
	clrGreen = SDL_MapRGB(screen->format,	0, 255,   0);

	clrGreenSDL.g = 210;
	
	time_t seconds;
	seconds = time(0);
	srand(seconds);

	return 0;
}

int game() {
	// This variable makes the quit loop work, frame counts the amount of frames that have passed
	bool quit = false;
	int frame = 0;
	
	// Temporary Rectangle
	SDL_Rect tRect;
	tRect.x = 0;
	tRect.y = 0;
	tRect.w = 180;
	tRect.h = 25;
	
	// The timer to maintain a steady 60 FPS, and a timer to calculate the actual FPS
	Timer fpsTimer;
	Timer gameTime;
	
	gameTime.start_timer();

	p1Bat.set_ai(true);
	p2Bat.set_ai(true);

	while (!quit) {
		// Here we reset the FPS caclulator's variables
		if (gameTime.get_ticks() > 10000) {
			gameTime.start_timer();
			frame = 0;
		}
		fpsTimer.start_timer();
		
		// This part makes sure that when you want to exit, the game exits
		while (SDL_PollEvent(&event)) {
			if (event.type == SDL_QUIT) {
				quit = true;
			}
			
			if (event.type == SDL_KEYDOWN) {
				if (event.key.keysym.sym == SDLK_1) {
					p1Bat.set_ai(!p1Bat.ai_enabled());
				}
				if (event.key.keysym.sym == SDLK_2) {
					p2Bat.set_ai(!p2Bat.ai_enabled());
				}
			}
		}
		
		keystates = SDL_GetKeyState(NULL);

		if (keystates[SDLK_r]) {
			ball.restart();
			score.points_reset();
		}

		SDL_GetMouseState(&mouseX, &mouseY);
		
		p1Bat.bat_update(keystates, 0x00FFCC);
		p2Bat.bat_update(keystates, 0x00FFCC);

		ball.update(keystates);
		if (ball.x_pos() + (BALL_WIDTH / 2) == SCR_WIDTH) {
			score.point_p1();
			ball.restart();
		} else if(ball.x_pos() - (BALL_WIDTH / 2) == 0) {
			score.point_p2();
			ball.restart();
		}

		score.display_score();
		
		/*
		// Display the current angle of the Ball
		std::stringstream textBallAngle;
		textBallAngle << "Ball Angle: " << ball.give_angle();
		imageBallAngle = TTF_RenderText_Solid(defaultFont, textBallAngle.str().c_str(), clrGreenSDL);
		SDL_FillRect(screen, &tRect, clrBlack);
		apply_surface(imageBallAngle, screen, 0, 0);
		*/

		// Calculate and update the FPS
		std::stringstream fps;
		fps << "Pong | FPS: " << ceil (frame / (gameTime.get_ticks() / 1000.f)) <<" | ( " << mouseX << " , " << mouseY << " )" << " " << time(NULL);
		SDL_WM_SetCaption(fps.str().c_str(), NULL);
		
		//Draw field draw's the dashes in the mid D:
		draw_field();

		// Sync
		SDL_Flip(screen);

		// Make the trial by fading some pixels around the ball to black slowly
		fade_screen(ball.x_pos(), ball.y_pos(), (BALL_WIDTH / 2) + BALL_TRAIL_LENGTH);

		// Clear the Top-Bar
		tRect.x = 0;
		tRect.y = 0;
		tRect.w = SCR_WIDTH;
		tRect.h = 38;
		SDL_FillRect(screen, &tRect, 0x000000);
		
		// Increase the frame counter XD
		frame++;
		
		// If the current frame took less than 1000 / FRAMES_PER_SECOND,
		// it delays the frame for a given amount of time
		if (fpsTimer.get_ticks() < 1000 / FRAMES_PER_SECOND && FPS_CAP) {
			SDL_Delay((1000 / FRAMES_PER_SECOND) - fpsTimer.get_ticks());
		}
	}

	return 0;
}

int exit() {
	// Close 2 fonts, a surface, and the 2 libraries
	TTF_CloseFont(scoreFont);
	TTF_CloseFont(defaultFont);

	SDL_FreeSurface(imageBallAngle);
	SDL_FreeSurface(tSurface);
	
	TTF_Quit();
	SDL_Quit();

	return 0;
}

int main( int argc, char* args[] ) {
	// Init the main libraries of the game, and other stuff
	// If anything goes wrong, Init() returns more than 0, and the game will quit
	if (init() > 0) {
		return 1;
	}
	
	// Holds the game
	game();
	
	// Cleans up surfaces/fonts, shuts down lirbaries, etc.
	exit();

	return 0;
}